% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_informant_report.R
\name{get_informant_report}
\alias{get_informant_report}
\title{Get a table information report from an \emph{informant} object}
\usage{
get_informant_report(
  informant,
  size = "standard",
  title = ":default:",
  lang = NULL,
  locale = NULL
)
}
\arguments{
\item{informant}{\emph{The pointblank informant object}

\verb{obj:<ptblank_informant>} // \strong{required}

A \strong{pointblank} \emph{informant} object that is commonly created through the
use of the \code{\link[=create_informant]{create_informant()}} function.}

\item{size}{\emph{Size option for display-table report}

\verb{scalar<character>} // \emph{default:} \code{"standard"}

The size of the display table, which can be either \code{"standard"} (the
default, with a width of 875px), \code{"small"} (width of 575px), or, a pixel-
or percent-based width of your choosing (supply an integer value for the
width in pixels, or values with \code{"px"} or \code{"\%"} appended, like \code{"75\%"},
\code{"500px"}, etc.).}

\item{title}{\emph{Title customization options}

\verb{scalar<character>} // \emph{default:} \code{":default:"}

Options for customizing the title of the report. The default is the keyword
\code{":default:"} which produces generic title text that refers to the
\strong{pointblank} package in the language governed by the \code{lang} option.
Another keyword option is \code{":tbl_name:"}, and that presents the name of the
table as the title for the report. If no title is wanted, then the
\code{":none:"} keyword option can be used. Aside from keyword options, text can
be provided for the title and \code{glue::glue()} calls can be used to construct
the text string. If providing text, it will be interpreted as Markdown text
and transformed internally to HTML. To circumvent such a transformation,
use text in \code{\link[=I]{I()}} to explicitly state that the supplied text should not be
transformed.}

\item{lang}{\emph{Reporting language}

\verb{scalar<character>} // \emph{default:} \code{NULL} (\code{optional})

The language to use for the \emph{information report}. By default, \code{NULL} will
create English (\code{"en"}) text. Other options include French (\code{"fr"}), German
(\code{"de"}), Italian (\code{"it"}), Spanish (\code{"es"}), Portuguese (\code{"pt"}), Turkish
(\code{"tr"}), Chinese (\code{"zh"}), Russian (\code{"ru"}), Polish (\code{"pl"}), Danish
(\code{"da"}), Swedish (\code{"sv"}), and Dutch (\code{"nl"}). This \code{lang} option will
override any previously set language setting (e.g., by the
\code{\link[=create_informant]{create_informant()}} call).}

\item{locale}{\emph{Locale for value formatting}

\verb{scalar<character>} // \emph{default:} \code{NULL} (\code{optional})

An optional locale ID to use for formatting values in the
\emph{information report} summary table according the locale's rules. Examples
include \code{"en_US"} for English (United States) and \code{"fr_FR"} for French
(France); more simply, this can be a language identifier without a country
designation, like \code{"es"} for Spanish (Spain, same as \code{"es_ES"}). This
\code{locale} option will override any previously set locale value (e.g., by the
\code{\link[=create_informant]{create_informant()}} call).}
}
\value{
A \strong{gt} table object.
}
\description{
We can get a table information report from an informant object that's
generated by the \code{\link[=create_informant]{create_informant()}} function. The report is provided as a
\strong{gt} based display table. The amount of information shown depends on the
extent of that added via the use of the \verb{info_*()} functions or through
direct editing of a \strong{pointblank} YAML file (an informant can be written
to \strong{pointblank} YAML with \verb{yaml_write(informant = <informant>, ...)}).
}
\section{Function ID}{

7-2
}

\examples{
# Generate an informant object using
# the `small_table` dataset
informant <- create_informant(small_table)

# This function creates some information
# without any extra help by profiling
# the supplied table object; it adds
# the sections 'table' and columns' and
# we can print the object to see the
# table information report

# Alternatively, we can get the same report
# by using `get_informant_report()`
report <- get_informant_report(informant)
class(report)

}
\seealso{
Other Incorporate and Report: 
\code{\link{incorporate}()}
}
\concept{Incorporate and Report}
