% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/object_ops.R
\name{x_read_disk}
\alias{x_read_disk}
\title{Read an \emph{agent}, \emph{informant}, \emph{multiagent}, or table scan from disk}
\usage{
x_read_disk(filename, path = NULL, quiet = FALSE)
}
\arguments{
\item{filename}{\emph{File name}

\verb{scalar<character>} // \strong{required}

The name of a file that was previously written by \code{\link[=x_write_disk]{x_write_disk()}}.}

\item{path}{\emph{File path}

\verb{scalar<character>} // \emph{default:} \code{NULL} (\code{optional})

An optional path to the file (combined with \code{filename}).}

\item{quiet}{\emph{Inform (or not) upon file writing}

\verb{scalar<logical>} // \emph{default:} \code{FALSE}

Should the function \emph{not} inform when the file is written?}
}
\value{
Either a \code{ptblank_agent}, \code{ptblank_informant}, or a
\code{ptblank_tbl_scan} object.
}
\description{
An \emph{agent}, \emph{informant}, \emph{multiagent}, or table scan that has been written to
disk (with \code{\link[=x_write_disk]{x_write_disk()}}) can be read back into memory with the
\code{x_read_disk()} function. For an \emph{agent} or an \emph{informant} object that has
been generated in this way, it may not have a data table associated with it
(depending on whether the \code{keep_tbl} option was \code{TRUE} or \code{FALSE} when
writing to disk) but it should still be able to produce reporting (by
printing the \emph{agent} or \emph{informant} to the console or using
\code{\link[=get_agent_report]{get_agent_report()}}/\code{\link[=get_informant_report]{get_informant_report()}}). An \emph{agent} will return an
x-list with \code{\link[=get_agent_x_list]{get_agent_x_list()}} and yield any available data extracts with
\code{\link[=get_data_extracts]{get_data_extracts()}}. Furthermore, all of an \emph{agent}'s validation steps will
still be present (along with results from the last interrogation).
}
\details{
Should a written-to-disk \emph{agent} or \emph{informant} possess a table-prep formula
or a specific in-memory tablewe could use the \code{\link[=interrogate]{interrogate()}} or
\code{\link[=incorporate]{incorporate()}} function again. For a \emph{data quality reporting} workflow, it
is useful to \code{\link[=interrogate]{interrogate()}} target tables that evolve over time. While the
same validation steps will be used, more can be added before calling
\code{\link[=interrogate]{interrogate()}}. For an \emph{information management} workflow with an \emph{informant}
object, using \code{\link[=incorporate]{incorporate()}} will update aspects of the reporting such as
table dimensions, and info snippets/text will be regenerated.
}
\section{Examples}{

\subsection{A: Reading an agent from disk}{

The process of developing an agent and writing it to disk with the
\code{\link[=x_write_disk]{x_write_disk()}} function is explained in that function's documentation.
Suppose we have such a written file that's named \code{"agent-small_table.rds"},
we could read that to a new agent object with \code{x_read_disk()}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{agent <- x_read_disk("agent-small_table.rds")
}\if{html}{\out{</div>}}
}

\subsection{B: Reading an informant from disk}{

If there is an informant written to disk via \code{\link[=x_write_disk]{x_write_disk()}} and it's named
\code{"informant-small_table.rds"}. We could read that to a new informant object
with \code{x_read_disk()}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{informant <- x_read_disk("informant-small_table.rds")
}\if{html}{\out{</div>}}
}

\subsection{C: Reading a multiagent from disk}{

The process of creating a multiagent and writing it to disk with the
\code{\link[=x_write_disk]{x_write_disk()}} function is shown in that function's documentation. Should
we have such a written file called \code{"multiagent-small_table.rds"}, we could
read that to a new multiagent object with \code{x_read_disk()}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{multiagent <- x_read_disk("multiagent-small_table.rds")
}\if{html}{\out{</div>}}
}

\subsection{D: Reading a table scan from disk}{

If there is a table scan written to disk via \code{\link[=x_write_disk]{x_write_disk()}} and it's named
\code{"tbl_scan-storms.rds"}, we could read it back into R with \code{x_read_disk()}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{tbl_scan <- x_read_disk("tbl_scan-storms.rds")
}\if{html}{\out{</div>}}
}
}

\section{Function ID}{

9-2
}

\seealso{
Other Object Ops: 
\code{\link{activate_steps}()},
\code{\link{deactivate_steps}()},
\code{\link{export_report}()},
\code{\link{remove_steps}()},
\code{\link{set_tbl}()},
\code{\link{x_write_disk}()}
}
\concept{Object Ops}
